#include "Moose.h"

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)


void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0,  -0.0,  -0.0, -0.00, 0.0,    0.0,   0.0, 0.0, 0.0, 0.0, -0.0, -0.0, -0.0, -0.0, -0.0, 0.0};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.12 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0, 0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.12 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


MOOSE::MOOSE (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hCone = oapiLoadMeshGlobal( "Moose/Moose");
	hParaPack = oapiLoadMeshGlobal( "Moose/ParaPack");
	hDrogue = oapiLoadMeshGlobal( "Moose/MooseDrogue");
	hChute = oapiLoadMeshGlobal( "Moose/MooseChute");
	hChuteDeflated = oapiLoadMeshGlobal( "Moose/MooseChuteDeflated");

	hDefSuit = oapiLoadMeshGlobal("gem-eva1");
	tex   = oapiRegisterExhaustTexture ("exhaust_atrcs");

	banimsset = false;
	defaultchecked = false;
	double RCSMass = 1.0;
	EjectMass = 7.5;
	RetroMass = 12.5;
	RetroMade = false;
	Burning = false;
	stage = 0;
	dragset = 0.0;
	drymass = 100.0;
	xscale = 1.5;
	yscale = 1.5;
	zscale = 1.5;
	PayRemoved = true;
	bExpands = true;
	CreateVariableDragElement (&dragset, 36.0, _V(0, 6.0, 0.0));

	//                alignment      position      func    chord S   AR   
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0.75, 0.75, 1);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0.75, 0.75, 1);
	// vertical stabiliser and body lift and drag components


	defaultsuit = false;
//	SetDefSuit();
	
	//		     				child       pos        dir       rot      handle
	Seatpos = CreateAttachment (false, OFS_SEAT,   _V(0,-1,0), _V(0,0,1), "SEAT");


	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if(( strstr( classname, "EJECTSEAT" ) ) || ( strstr( classname, "NOSUITEJECTSEAT" ) )) {
		RCSMass = 0.0;
		RetroMass = 0.0;
		bExpands = false;
	}
	else if(( strstr( classname, "PARACHUTIST" ) ) || ( strstr( classname, "NOSUITPARACHUTIST" ) )) {
		RCSMass = 0.0;
		RetroMass = 0.0;
		EjectMass = 0.0;
		drymass = 10.0;
		stage = 1;
	}
/*	else if( strstr( classname, "MOOSEMMU" ) ) {
			xscale = 1.75;
			yscale = 1.75;
			zscale = 1.75;
		}
	else if(( strstr( classname, "MOOSE2001" ) ) || ( strstr( classname, "EJECTSEAT2001" ) )){
			yscale = 1.4;
		}
	else if( strstr( classname, "MOOSEDG3" ) ) {
			xscale = 1.25;
			yscale = 1.25;
			zscale = 1.25;
		}
*/


	ph_main = CreatePropellantResource( RCSMass );
	ph_eject = CreatePropellantResource (EjectMass);
	ph_retro = CreatePropellantResource (RetroMass);


}

void MOOSE::LoadState (FILEHANDLE scn, void *vs)
{
	char *line;
//	int i=0;
      
	ClearMeshesEx();
	while (oapiReadScenario_nextline (scn, line)) 
	{
  		if (!strnicmp (line, "PAYLOAD", 7))
		{
			if (!defaultsuit)
				ParsePayloadString( line );
		}

		else if ( !strnicmp(line, "CONFIGURATION",13))
		{
			sscanf (line+13, "%d", &stage);
		}
		else if (!strnicmp (line, "EXPAND", 6))
		{
			sscanf (line+6, "%lf", &expand_proc);
		}



		else 
		{
			ParseScenarioLineEx (line, vs);
		}

	}

	if (stage > 1) {
		SetPMI (_V(22.28, 22.15, 24.09));
		expand_proc = 0.0;
		bExpands = false;
	}
	SetStage();
	SetAnims();
	Draw();

}

void MOOSE::SaveState (FILEHANDLE scn)
{
	char cbuf[256];

	SaveDefaultState (scn);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	sprintf (cbuf, "%0.4f", expand_proc);
	oapiWriteScenario_string (scn, "EXPAND", cbuf);


	/* save payload lines */
	if (!defaultsuit)
		SaveDefaultStateEx(scn);
}

void MOOSE::SetDefSuit()
{
		defaultchecked = true;

		char checkname[512];

		strcpy (checkname, GetName()); 
		strcat (checkname, "-Suit");
		hVessel=oapiGetVesselByName(checkname);
		if (hVessel != 0) return;					// There's already a suit of this name built;
												// probably a saved scenario with the suit already
												// jettisoned.  Don't make another.

		char paystr[80];
		char classname[30];
		int i;
		strncpy( classname, GetClassName(), 29 );
		classname[29]=0;
		for( i = 0 ; i < strlen( classname ) ; i++ ) {
			classname[i] = toupper(classname[i]);
		}
		if( strstr( classname, "MOOSEMMU" ) ) {
			strncpy(paystr, "mmu nasa_mmu 0 0.0 0.12 212.0 0.0 1.0", 79);
		}
		else if(( strstr( classname, "MOOSE2001" ) ) || ( strstr( classname, "EJECTSEAT2001" ) )){
			strncpy(paystr, "2001/2001Astro 2001/Astro2001 0 -0.2 0.15 130.0 0.0 1.0", 79);
			PayRemoved = false;
		}
		else if( strstr( classname, "MOOSEDG3" ) ) {
			strncpy(paystr, "DGIIIMmu DGIIIMmu 0 0.0 .05 200.0 0.0 1.0", 79);
		}
		else if(( strstr( classname, "MOOSE" ) ) || ( strstr( classname, "PARACHUTIST" ) ) || ( strstr( classname, "EJECTSEAT" ) )){
			strncpy(paystr, "gem-eva1 gemini_leva  0.0  0.0 0.15 150.0  0.0  1.0", 79);
			PayRemoved = false;
		}
		else return;

		char prefix[512];

		strcpy (prefix, "PAYLOAD ");
		strcat (prefix, GetName()); 
		strcat (prefix, "-Suit ");
		strcat (prefix, paystr);

		ParsePayloadString(prefix);
		defaultsuit = true;
		CheckCentralMessage();
		SetStage();
		SetAnims();
}


void MOOSE::SetStage ()
{

	VECTOR3 pos = _V( 0,0,0 );
	COMMON_MESH tmp;
//	ClearVariableDragElements();



	ClearMeshesEx();
	if (stage == 0) {
		tmp.MeshHandle(hCone);
		tmp.MeshPosition( pos);
		AddMeshEx( tmp);
	}
	else if (stage == 1) {
		tmp.MeshHandle(hParaPack);
		tmp.MeshPosition( pos);
		AddMeshEx( tmp);
	}
	else if (stage == 2) {
		tmp.MeshHandle(hDrogue);
		tmp.MeshPosition( pos);
		AddMeshEx( tmp);
		dragset = 0.05;
		SetPitchMomentScale (0.0);
		SetBankMomentScale (0.0);
	}
	else if (stage == 3) {
		tmp.MeshHandle(hChute);
		tmp.MeshPosition( pos);
		AddMeshEx( tmp);
		dragset = 1.0;
		SetPitchMomentScale (0.0);
		SetBankMomentScale (0.0);
	}
	else if (stage == 4) {
		tmp.MeshHandle(hChuteDeflated);
		tmp.MeshPosition( pos);
		AddMeshEx( tmp);
		SetTouchdownPoints(_V(0.0,-0.0,10.0),_V(-10.0,-0.0,-4.0),_V(10.0,-0.0,-4.0));
	}
	
}
void MOOSE::SetAnims ()
{

	Draw();
	if (!banimsset) {
		banimsset = true;
	
		anim_expand = CreateAnimation (0.0);

		if (stage < 2) {

		static UINT ExpandGrp[1] = {1};
//		static UINT RetroGrp[2] = {0,1};

//		static MGROUP_ROTATE Retro (0, RetroGrp, 2,
//			_V(0.0,0.0,-0.45), _V(1,0,0), (float)(68*RAD));
//		AddAnimationComponent (anim_expand, 0.05, 0.9, &Retro);

	//	static MGROUP_TRANSLATE trans (0, ExpandGrp, 2,
	//		_V( 0.0, 0.0, 0.5));
	//	AddAnimationComponent (anim_expand, 0.68, 0.7, &trans);

		static MGROUP_SCALE Expand (0, ExpandGrp, 1,
			_V(-0.0095, 0.0312, -0.2465), _V(100.0*xscale,95.0*yscale,75.0*zscale));
		AddAnimationComponent (anim_expand, 0.05, 0.95, &Expand);
		}
	}

	SetAnimation (anim_expand, expand_proc);

	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	VECTOR3 mesh_dir=_V(0,0.50,-0.4);
	SetCameraOffset(mesh_dir);

	
	double MainThrust = 0.0;
	double HoverThrust = 0.0;
	RetroThrust = 2000;
	double EjectThrust = 18000;
	double AttThrust = 5;
	EXPAND_SPEED = 0.5;
	expand_proc = 0.0;
	trans_proc = 1.0;
	numevas = 0;
	Expanding = false;

	SetEmptyMass( drymass+CalculateMass());
//	if (defaultsuit) SetEmptyMass (350.0);


	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.5, 5, 250, 0.3, 0.5, 16, 2.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};

	PARTICLESTREAMSPEC exhaust_retro = {
		0, 1.2, 40, 250, 0.1, 0.2, 42, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};
	exhaust_retro.tex = oapiRegisterParticleTexture ("Contrail2");
	SetRotDrag (_V(1.7,1.7,1.3)); // angular drag
//	SetEnableFocus(FALSE);

/*
	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);
*/
/*	PARTICLESTREAMSPEC rps = {
		0, 4, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 3e9, 12e9
	};
	AddReentryStream (&rps);
*/

//	ClearVariableDragElements();
//	CreateVariableDragElement (&dragset, 20.0, _V(0, 20, 0));

	SetSize (2);
	SetTouchdownPoints(_V(0.0,-2.0,10.0),_V(-10.0,-2.0,-4.0),_V(10.0,-2.0,-4.0));
	SetPMI (_V(2.28, 2.15, 4.09));
	SetCrossSections (_V(1.3, 1.3, 3.3));
	SetCW (0.7, 0.7, 0.9, 0.9);
	SetRotDrag (_V(20.0,20.0,15.0));
	SetPitchMomentScale (-1e-4);
	SetBankMomentScale (-1e-4);
	SetDefaultPropellantResource( ph_retro );
//	Burning = false;

	retro_engine[0] = CreateThruster (_V(0,-0.25,0.0),    _V( 0.0,1.0,0.0), 
		RetroThrust, ph_retro, 2500, 2500);
	AddExhaust(retro_engine[0],1.0,0.1, _V(0,-0.25,0), _V(0,-1,0),tex);
	thg_retro = CreateThrusterGroup(retro_engine,1,THGROUP_HOVER);
	AddExhaustStream (retro_engine[0], &exhaust_retro);

	eject_engine[0] = CreateThruster (_V(0,-0.25,0.0),    _V( 0.0,1.0,0.0), 
		EjectThrust, ph_eject, 2500, 2500);
	AddExhaust(eject_engine[0],1.0,0.1, _V(0,-0.25,0), _V(0,-1,0),tex);
	thg_eject = CreateThrusterGroup(eject_engine,1,THGROUP_HOVER);
	AddExhaustStream (eject_engine[0], &contrail);

//	retex = oapiRegisterReentryTexture("reentry");
//	SetReentryTexture(retex,6e5,1,1);



// 5 Dock #3



/*
	th_main[0] = CreateThruster (_V(0,0.0,-15.5),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, 4e4, 4e5);

	th_main[1] = CreateThruster (_V(-3.7,0.0,-11.5), _V( 0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	th_main[2] = CreateThruster (_V( 3.7,0.0,-11.5), _V( -0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	thg_main = CreateThrusterGroup( th_main, 3, THGROUP_MAIN );
	AddExhaustStream (th_main[0], _V( 0.0,  0, -25.0), &exhaust_main);
	AddExhaustStream (th_main[1], _V(-2.6, 0.0,-20.0), &exhaust_main);
	AddExhaustStream (th_main[2], _V( 2.6, 0.0,-20.0), &exhaust_main);

	AddExhaust (th_main[0], 10.0, 1.0, tex);
	AddExhaust (th_main[1], 10.0, 0.5, tex);
	AddExhaust (th_main[2], 10.0, 0.5, tex);

	th_hover[0] = CreateThruster (_V(0,0,0), _V(0,1,0), HoverThrust, ph_main, 4e4, 4e5);
	thg_hover = CreateThrusterGroup (th_hover, 1, THGROUP_HOVER);

	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
*/
	// set of attitude thrusters (idealised). The arrangement is such that no angular
	// momentum is created in linear mode, and no linear momentum is created in rotational mode.

	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);

	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);

	th_att_rot[0] = CreateThruster (_V( 2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(-2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(-2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V( 2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0,-16), _V(0,0, 1), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0, 16), _V(0,0,-1), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+1, 1, THGROUP_ATT_BACK);


}

void MOOSE::SetClassCaps (FILEHANDLE cfg)
{

	CheckCentralMessage();

	SetStage();
	SetAnims();
}


void MOOSE::TimeStep( double simt )
{

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ; {
			SetStage();
			Draw();
		}
	}

	OBJHANDLE hV = GetAttachmentStatus(Seatpos);
	if (hV) {
		VESSEL *v = oapiGetVesselInterface (hV);
		double strappedmass = v->GetMass();
		SetEmptyMass (drymass+strappedmass+CalculateMass());
		defaultchecked = true;
	}

	if (!defaultchecked)
		if (GetMass() == (drymass + GetPropellantMass(ph_main) + GetPropellantMass(ph_eject) + GetPropellantMass(ph_retro)))		// No suit; just pack and propellant
			SetDefSuit();
/*
	if (!PayRemoved) {
		PayRemoved = true;
		defaultchecked = true;
		OverridePayloadVelocity(_V(0,0,0));
		depV = DeployPayload();
		SetStage();
		Draw();
		SetEmptyMass( drymass+CalculateMass());
		if (depV)
			StrapUnstrap();
	}
*/
	if (RetroMade) {
		if (GetThrusterGroupLevel (thg_retro) > 0.0) 
			Burning = true;
		if (Burning) 
			SetThrusterGroupLevel (thg_retro,1.0);  // Act as a solid motor (burn to depletion)
		if  (GetPropellantMass(ph_retro) == 0) {
			Expanding = true;
		}
	} else {
		if (GetThrusterGroupLevel (thg_eject) > 0.0) 
			Burning = true;
		if (Burning) 
			SetThrusterGroupLevel (thg_eject,1.0);  // Act as a solid motor (burn to depletion)
		if  (GetPropellantMass(ph_eject) == 0) {
			SetThrusterGroupLevel(THGROUP_HOVER,0.0);
			thg_retro = CreateThrusterGroup(retro_engine,1,THGROUP_HOVER);
			Burning = false;
			RetroMade = true;
		}
	}


	if ((stage < 4) && (GetAltitude() < 10000.0)) {
		if (stage == 3)
			if (GetAltitude() < 70.0) {
				GetStatus(vs);
				if ((vs.status == 1) || (GetAltitude() < 6.0)) {
					stage = 4;
					SetStage();
					Draw();
	//				StrapUnstrap();
			hV = GetAttachmentStatus(Seatpos);
			if (hV) {
				AddForce (_V(0, 0, 10*GetMass()), _V(0, 0, 0));
				StrapUnstrap();
					}
				}
		}
		if (GetAltitude() < 2000.0)
			if ((stage == 1) || (stage == 2))
				if (GetMachNumber() < 0.6) {
					stage = 3;
					SetPMI (_V(22.28, 22.15, 24.09));
					SetStage();
					Draw();
				}
		if ((stage == 0) && (GetPropellantMass(ph_eject) == 0) && (GetThrusterGroupLevel (thg_eject) == 0) 
			&& (GetThrusterGroupLevel (thg_retro) == 0))
			if (GetMachNumber() < 1.0) {
				stage++;
				stage++;
				SetPMI (_V(22.28, 22.15, 24.09));
				SetAnimation (anim_expand, 0.0);
				char name[256];
				strcpy (name, GetName()); strcat (name, "-Seat");
				GetStatus(vs);
				ClearThrusterDefinitions();
				ClearPropellantResources();
				if (expand_proc == 0.0)
					hV = oapiCreateVessel (name, "MOOSE/JettisonedSeat", vs);
				else hV = oapiCreateVessel (name, "MOOSE/JettisonedExpandedSeat", vs);
//				VESSEL *remotevessel = oapiGetVesselInterface(hV);

				SetStage();
				Draw();
	
/*				anim_expand2 = remotevessel->CreateAnimation (0.0);
				static UINT ExpandGrp2[1] = {1};
				static MGROUP_SCALE Expand2 (0, ExpandGrp2, 1,
					_V(-0.0095, 0.0312, -0.2465), _V(100.0*xscale,95.0*yscale,75.0*zscale));
				remotevessel->AddAnimationComponent (anim_expand2, 0.05, 0.95, &Expand2);
				remotevessel->SetAnimation (anim_expand2, expand_proc);
*/
			}
	}


	if ((stage == 0) && (bExpands)) {
		SetAnimation (anim_expand, expand_proc);
		double da = oapiGetSimStep() * EXPAND_SPEED;
		if (Expanding) {
			expand_proc = expand_proc + da;
		}
		if (expand_proc >= 1.0) {
			Expanding = false;
			expand_proc = 1.0;
		}
	}

}

void MOOSE::StrapUnstrap()
{

			VECTOR3 gpos, grms, pos, dir, rot;
			Local2Global (OFS_SEAT, grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
				OBJHANDLE hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					
//					sprintf(oapiDebugString(),"Distance is too far to strap in!");

					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BUTT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!
							// check whether satellite is currently clamped into payload bay
							if (hV == GetAttachmentStatus (Seatpos))
							{
								DetachChild (Seatpos);
//								sprintf(oapiDebugString(),"Unstrapped");
								oapiSetFocusObject (hV);
								SetEmptyMass (drymass);
							}
							else
							{
								AttachChild (hV, Seatpos, hAtt);
//								sprintf(oapiDebugString(),"Strapped in!");	
							}
							return;
						}
					}
				}
			
			}
}

int MOOSE::ConsumeKey ( const char *keystate)
{
	return 0;
}

int MOOSE::ConsumeBufferedKey ( DWORD dwKey, bool fPressed, const char *keystate)
{

	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
	}
	else {
		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // "Unfold"
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
			//	OperateDoors (door_status == DOOR_CLOSED || door_status == DOOR_CLOSING ?
			//		DOOR_OPENING : DOOR_CLOSING);
//				Expanding = true;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				OverridePayloadVelocity(_V(0,0,0));
//				depV = DeployPayload();
				SetStage();
				Draw();
//				SetEmptyMass( 100.0+CalculateMass());
//				if (depV) {
//					hVessel=oapiGetVesselByName(depV->GetName());
//					oapiSetFocusObject(hVessel);
//				}
				if ((RetroMade) && (Burning)) {							// Blow off the nozzle, causing the motor to smolder uselessly
					SetThrusterIsp(retro_engine[0],25.0);
					SetThrusterMax0(retro_engine[0],RetroThrust/100);
				}

			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_S)) { // "Strap/Unstrap" 
			if (oapiAcceptDelayedKey (OAPI_KEY_S, 1.0)) {
				OverridePayloadVelocity(_V(0,0,0));
				depV = DeployPayload();
				SetEmptyMass( drymass+CalculateMass());
				if (depV) {
					hVessel=oapiGetVesselByName(depV->GetName());
					oapiSetFocusObject(hVessel);
				}
				else
					StrapUnstrap();
				SetStage();
				Draw();
			}
			return 1;
		}

/*
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
			

				VESSELSTATUS vs;
				OBJHANDLE hvessel;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = _V(3.5,0.0,17.25);
				Local2Rel (ofs, vs.rpos);
				char name[256];
				numevas++;
				sprintf(name, "EVA-%d", numevas);
	
				VECTOR3 vel = _V(0,0.2,1.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				VESSEL::Create (name, "EVA", vs);
				hvessel=oapiGetVesselByName(name);
				oapiSetFocusObject(hvessel);
	
			}
			return 1;
		}
*/
	}
		return 0;
}



/* basic vessel class setup */

void MOOSE::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void MOOSE::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void MOOSE::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void MOOSE::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void MOOSE::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetStage( );
}

void MOOSE::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetStage( );
}

int MOOSE::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new MOOSE (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (MOOSE*)vessel;
}
